@@ -54,7 +54,7 @@ var _ = Describe("command line tool flags", func() {
 	Context("yaml command", func() {
 		Context("creating yaml output", func() {
 			It("should not create YAML output that is not valid", func() {
-				filename := createTestFile(`{"a": ",", "foo": {"bar":"*"}}`)
+				filename := createTestFile(`{"a": ",", "foo": {"bar": "*", "dash": "-"}}`)
 				defer os.Remove(filename)
 
 				out, err := dyff("yaml", filename)
@@ -63,6 +63,7 @@ var _ = Describe("command line tool flags", func() {
 a: ","
 foo:
   bar: "*"
+  dash: "-"
 
 `))
 			})
@@ -534,7 +535,7 @@ yaml.map.whitespaces
 
 		It("should show a report when filtering and a document has been removed between inputs", func() {
 			expected := `
-spec.replicas  (Deployment/default/test)
+spec.replicas  (apps/v1/Deployment/test)
   ± value change
     - 2
     + 3
@@ -579,6 +580,17 @@ spec.replicas  (Deployment/default/test)
 			Expect(err).ToNot(HaveOccurred())
 			Expect(out).To(BeEquivalentTo("\n"))
 		})
+
+		It("should ignore the 'whitespace only' changes", func() {
+			out, err := dyff("between",
+				"--omit-header",
+				"--ignore-whitespace-changes",
+				assets("issues", "issue-222", "from.yml"),
+				assets("issues", "issue-222", "to.yml"))
+
+			Expect(err).ToNot(HaveOccurred())
+			Expect(out).To(BeEquivalentTo("\n"))
+		})
 	})
 
 	Context("last-applied command", func() {
